@@ -57,7 +57,7 @@
     html_logo_url = "https://raw.githubusercontent.com/tiby312/broccoli/master/assets/logo.png",
     html_favicon_url = "https://raw.githubusercontent.com/tiby312/broccoli/master/assets/logo.png"
 )]
-#![no_std]
+//#![no_std]
 
 #[cfg(doctest)]
 mod test_readme {
@@ -828,14 +828,9 @@ impl<'a, T: Aabb> Tree<'a, T> {
     pub fn for_all_intersect_rect_mut<'b>(
         &'b mut self,
         rect: &Rect<T::Num>,
-        mut func: impl FnMut(PMut<'b, T>),
+        func: impl FnMut(PMut<'b, T>),
     ) {
-        queries::rect::for_all_intersect_rect_mut(
-            default_axis(),
-            self.vistr_mut(),
-            rect,
-            move |a| (func)(a),
-        );
+        queries::rect::for_all_intersect_rect_mut(default_axis(), self.vistr_mut(), rect, func);
     }
 
     /// # Examples
